#version 430 core
in vec3 Positions;	
in vec3 Normals;
in vec2 TexCoords;

out vec4 gl_FragColor;

uniform sampler2D textureDiffuse1;
uniform float brightness;
uniform vec3 cameraPosition;

vec3 colorLight = vec3(1, 1, 1);					// uniform
vec3 colorLightPoint = vec3(1, 1, 1);				// uniform
vec3 colorLightDir = vec3(1, 1, 1);					// uniform

vec3 positionLight = vec3(0.0, 1.0, -4.0);			// uniform

vec3 directionLight = vec3(-1.0, -1.0, -1.0);		// uniform

float Ka = 0.30;									// uniform
float Kd = 0.58;									// uniform
float Ks = 0.0;										// uniform
int alpha = 10;										// uniform                           

//uniform int levels;
int levels = 5;

struct PointLight
{
	vec3 colorLight;
	vec3 positionLight;

	float diffuse;
	float ambient;
	float specular;
};

struct DirLight
{
	vec3 colorLight;
	vec3 directionLight;

	float diffuse;
	float ambient;
	float specular;
};

struct SpotLight
{
	vec3 colorLight;
	vec3 posLight;
	
	vec3 attenuationLight;
	
	float cutOff;
	float outerCutOff;

	float diffuse;
	float ambient;
	float specular;
};

uniform PointLight pointLight;
uniform DirLight dirLight;
uniform SpotLight spotLight;


// Functions
vec3 CalcPointLight (PointLight light, vec3 normal, vec3 fragPos, vec3 ViewDir);
vec3 CalcDirLight (DirLight light, vec3 normal, vec3 ViewDir);
vec3 CalcSpotLight (SpotLight light, vec3 normal, vec3 fragPos, vec3 ViewDir);

void main() {
	vec3 norm = normalize(Normals);
	vec3 viewDir = normalize(cameraPosition - Positions);

	// Texture
	vec3 textureColor = texture(textureDiffuse1, TexCoords).rgb;

	// Ambient
	vec3 FragColor = vec3(textureColor * colorLightDir * (Ka * brightness));

	// Calculate every light	
	FragColor += CalcDirLight(dirLight, norm, viewDir) + (colorLightDir * (Ka * brightness) * vec3(0.0, 0.0, 0.0));
	FragColor += CalcPointLight(pointLight, norm, Positions, viewDir);
	//FragColor += CalcSpotLight(spotLight, norm, Positions, viewDir);
		
	gl_FragColor = vec4(FragColor, 1.0);
}

// Directional light
vec3 CalcDirLight (DirLight light, vec3 normal, vec3 ViewDir){
	light.colorLight = colorLightDir;
	light.directionLight = directionLight;
	
	vec3 pointToLight = normalize(-light.directionLight);	
	float cosTheta = max(dot(normal, pointToLight), 0.0);	
	
	vec3 reflectDir = reflect(-pointToLight, normal);
	float spec = pow(max(dot(ViewDir, reflectDir), 0.0), alpha);
	
	// Cel shading diffuse
	float discreteFactor = floor(cosTheta * levels) / levels;
	light.diffuse = Kd * discreteFactor;

	// Cel shading specular
	discreteFactor = floor(spec * levels) / levels;
	light.specular = Ks * discreteFactor;

	return ((light.diffuse * texture(textureDiffuse1, TexCoords).rgb) + light.specular) * light.colorLight;
}

// Point light
vec3 CalcPointLight (PointLight light, vec3 normal, vec3 fragPos, vec3 ViewDir){
	
	light.colorLight = colorLightPoint;
	light.positionLight = positionLight;
	
	vec3 pointToLight = normalize(light.positionLight - Positions);	
	float cosTheta = max(dot(normal, pointToLight), 0.0);
	
	vec3 reflectDir = reflect(-pointToLight, normal);
	float spec = pow(max(dot(ViewDir, reflectDir), 0.0), alpha);

	// Cel shading diffuse
	float discreteFactor = floor(cosTheta * levels) / levels;
	light.diffuse = Kd * discreteFactor;

	// Cel shading specular
	discreteFactor = floor(spec * levels) / levels;
	light.specular = Ks * discreteFactor;
	
	float distance = length(light.positionLight - fragPos);
	float attenuation = 1.0f / (1 + 0.4f * distance + 0.1 * (distance*distance));

	return ((light.diffuse * texture(textureDiffuse1, TexCoords).rgb) + light.specular) * light.colorLight * attenuation;
}

// Spotlight
vec3 CalcSpotLight(SpotLight light, vec3 normal, vec3 fragPos, vec3 ViewDir){
	light.colorLight = colorLightPoint;
	light.posLight = cameraPosition;
		
	vec3 pointToLight = normalize(light.posLight - fragPos);
	
	float cosTheta = clamp(dot(pointToLight, normal), 0, 1);
	
	vec3 reflectDir = normalize(reflect(-pointToLight, normal));
	float spec = pow(max(dot(ViewDir, reflectDir), 0.0), alpha);

	float theta = dot(-pointToLight, normalize(-cameraPosition));
	float epsilon = light.cutOff - light.outerCutOff;
	float intensity = clamp((theta - light.outerCutOff) / epsilon, 0, 1);

	// Cel shading diffuse
	float discreteFactor = floor(cosTheta * levels) / levels;
	light.diffuse = Kd * discreteFactor;

	// Cel shading specular
	discreteFactor = floor(spec * levels) / levels;
	light.specular = Ks * discreteFactor;

	light.diffuse = Kd * cosTheta;
	light.specular = Ks * spec;

	float distance = length(light.posLight - fragPos);
	float attenuation = 1.0f / (1 + 0.4f * distance + 0.1 * (distance*distance));
	
	return (((light.diffuse * texture(textureDiffuse1, TexCoords).rgb) + light.specular) * intensity * light.colorLight * attenuation);
}
